Unit Process;

Interface

Uses

  SysUtils, Globals;

Const

  Version = '1.2';

Type

  TInfoCallback = Procedure( Const aMsg: String) Of Object;

  Procedure DoStuff( anInputFileName, anOutputFileName : String; anInfoCallBack: TInfoCallback);

Implementation

  Procedure DoStuff( anInputFileName, anOutputFileName : String; anInfoCallBack: TInfoCallback);
  Var
    TmpStr       : String;
    TmpFileName  : String;
    TemplateName : String;
    WaveFile     : File;
    TempFile     : File;
    TemplateFile : TextFile;
    OutputFile   : textfile;
    Buf          : byte;
    ByteCount    : Integer;
    FileCount    : Integer;
    ModuleFound  : Boolean;
  Begin
    Assert( Assigned  ( anInfoCallBack) , 'Bug - InfoCallback is needed.'         );
    Assert( FileExists( anInputFileName), 'Bug - an existing input file is needed');

    TmpFileName := ApplicationPath + 'Temp';
    ByteCount   := 0;
    ModuleFound := false;

    //First, we need to get the relevant file names

    anInfoCallBack( 'Simon''s Nord Modular Sample Generator.');
    anInfoCallBack( 'Modified to have some visuals by Jan Punter.');
    If pos( '.pch', anOutputFileName) = 0
    then anOutputFileName := anOutputFileName + '.pch';

    AssignFile( WaveFile, anInputFileName);
    AssignFile( TempFile, TmpFileName);
    Reset( WaveFile, 1);
    Rewrite( TempFile, 1);

    //Then we need to divide every byte by 2, so it is
    //in a range suitable for the control sequencer

    While Not EOF( WaveFile) Do
    begin
      BlockRead( WaveFile, Buf, 1);
      Buf := Buf div 2;
      BlockWrite( TempFile, Buf, 1);
      Inc( ByteCount);
      If ByteCount > 2048 Then
      Begin
        anInfoCallBack( 'Input file length greater than 2048 bytes... Truncating.');
                        //fixed... Said 'WriteLn' but is no longer a console app
        Exit;
      End;
    End;

    //Now close the files, and open the temp file for reading.
    CloseFile( WaveFile);
    CloseFile( TempFile);
    AssignFile( TempFile, TmpFileName);
    Reset( TempFile, 1);

    //Now let's parse the template patches... We're looking for
    //the string:  'Type=91' to show that we've found a control
    //sequencer, and 'P16=15' to show that we've finished filling
    //in the control parameters for the current module. ( P16 =
    //number of steps...)

    //Anything beginning with 'P' between these two lines will be the
    //next control parameter in the sequence.

    For FileCount := 1 To 4 Do
    Begin
      //added for additional diagnostics

      TemplateName := ApplicationPath + IntToStr( FileCount) + 'Template.pch';

      if not FileExists( TemplateName) then begin
        anInfoCallBack('Template file: ' + TemplateName + ' cannot be found!');
        anInfoCallBack('Please ensure the template files are in the application directory');
        CloseFile(TempFile);
        DeleteFile(TmpFileName);
        exit;
      end;

      AssignFile( OutputFile, ExtractFilePath( anOutputFileName) + IntToStr( FileCount) + ExtractFileName( anOutputFileName));
      Rewrite( OutputFile);

      AssignFile( TemplateFile, ApplicationPath + IntToStr( FileCount) + 'Template.pch');
      Reset( TemplateFile);

      While Not EOF( TemplateFile) Do
      Begin
        ReadLn( TemplateFile, TmpStr);

        //Find out if we've entered or left an area we're supposed to be modifying.

        If CompareText( 'Type=91', TmpStr) = 0
        Then ModuleFound := true;
        If CompareText( 'P16=15', TmpStr) = 0
        Then ModuleFound := false;

        If ModuleFound And ( TmpStr[ 1] = 'P') And ( Not EOF( TempFile))
        Then Begin

          //We've found a line that needs modifying... Replace the value

          BlockRead( TempFile, Buf, 1);
          Delete( TmpStr, pos( '=', TmpStr) + 1, 999);
          TmpStr := TmpStr + IntToStr( Buf);
        End;

        WriteLn( OutputFile, TmpStr);
      End;

      CloseFile( OutputFile);
      CloseFile( TemplateFile);

      // When there's no data left, we don't need to proceed to the next file
      If EOF( TempFile)
      Then Break;
    End;

    CloseFile( TempFile);
    DeleteFile( TmpFileName);
    anInfoCallback( 'Finished!');
  End;

end.
